
/*============================================================================
  File:     Transaction_examples.sql

  Date:     Nov 2006

  SQL Server Version: 9.0.2153
------------------------------------------------------------------------------
  Copyright (C) 2006 Bob Beauchemin, SQLskills, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Bob Beauchemin.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE pubs
go

select count(*) from jobs

-- 1. batches are parsed as atomic unit
-- how many rows inserted?
insert jobs values('job1', 10, 10)
insert jobs values('job2', 20, 20)
select * form authors
go

select count(*) from jobs
go

-- how many rows inserted?
-- GO ends the batch
insert jobs values('job1', 10, 10)
go
insert jobs values('job2', 20, 20)
go
select * form authors
go

select count(*) from jobs
go

-- 2. one session per connection 
-- sessions settings copied back to connection 
-- at end of batch
dbcc useroptions
set lock_timeout 10
dbcc useroptions
go

-- still set to 10
dbcc useroptions
go


-- 3. you can separate statements
-- by CR-LF, space, or semicolon

select * from authors select * from authors
go
select * from authors;select * from authors
go
select * from authors 
select * from authors
go

-- 4. semicolon required if WITH is not first statement
-- in a batch (also required on SEND, RECEIVE statements)
SELECT * FROM jobs
;WITH somecte 
AS
(
  SELECT au_id, au_lname FROM authors
)
SELECT * FROM somecte
go

-- 5. implicit transactions means transaction only ends
-- at commit or rollback

select count(*) from jobs

set implicit_transactions on
insert jobs values('job15', 10, 10)
insert jobs values('job16', 10, 10)
rollback
go

select count(*) from jobs

select @@trancount

set implicit_transactions off
-- now begin tran is required
insert jobs values('job15', 10, 10)
insert jobs values('job16', 10, 10)
rollback

begin tran
delete jobs where job_id = 4


-- 6. no true nested transactions
-- savepoints instead
-- how many jobs inserted?
begin tran
  insert jobs values('job22', 10, 10)
  begin tran
    insert jobs values('job23', 10, 10)
  rollback
commit

-- how many jobs inserted?
begin tran
  insert jobs values('job33', 10, 10)
  begin tran
    insert jobs values('job34', 10, 10)
  commit
rollback

-- how many jobs inserted?
begin tran
  insert jobs values('job44', 10, 10)
  save transaction foo
    insert jobs values('job45', 10, 10)
  rollback transaction foo
commit

-- 7. New transaction isolation levels
begin tran
  update authors set au_fname ='mary'
    where au_lname = 'smith'
-- dont commit yet
commit

-- now commit or rollback
rollback

-- 8. Nesting stored procs and transactions
create proc b
as
begin tran
print 'in proc b'
-- do some work
commit
go

create proc a 
as
begin tran
print 'in proc a'
execute b
commit
go

-- try it out
execute a

select @@trancount

-- now cause rollback in b
alter proc b
as
begin tran
print 'in proc b'
-- do some work
rollback
go

-- try it out, now
execute a











