using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;

namespace MARSDeadlock
{
  class Program
  {
    static void Main(string[] args)
    {

      /*
       * In order for a task to run under MARS, it must acquire the session
       * mutex. If the task is running under a transaction, it must then 
       * acquire the transaction mutex. This guarantees that only one task
       * is active at one time in a given session and a given transaction.
       * Once the required mutexes have been acquired, the task can execute.
       * When the task finishes, or yields in the middle of the request, 
       * it will first release transaction mutex followed by the session 
       * mutex in reverse order of acquisition. However, deadlocks can 
       * occur with these resources. 
       */ 
      /* In the following code example, two 
       * tasks, user request U1 and user request U2, are running in the 
       * same session.
       */

      SqlConnection conn = new SqlConnection("server=.;integrated security=sspi;database=AdventureWorks;multipleactiveresultsets=true;async=true");
      SqlCommand cmd1 = new SqlCommand("set transaction isolation level serializable;begin tran;insert Sales.SalesOrderDetailTemp EXEC Sales.SalesInsert;commit", conn);
      SqlCommand cmd2 = new SqlCommand("set transaction isolation level serializable;begin tran;select SalesOrderID from Sales.SalesOrderDetail;commit", conn);

      SqlDataReader rdr1 = null;
      SqlDataReader rdr2 = null;

      conn.Open();
      //SqlTransaction tx = conn.BeginTransaction();
      //cmd1.Transaction = tx;
      //cmd2.Transaction = tx;
      cmd2.CommandTimeout = 1000;
      rdr2 = cmd2.ExecuteReader();
      rdr2.Read();
      IAsyncResult r = cmd1.BeginExecuteNonQuery();

      while (!r.IsCompleted)
      {
        for (int i = 0; i < 100; i++)
        {
          rdr2.Read();
          Console.WriteLine("read one");
        }
        rdr2.Read();
        Console.Write(".");
        System.Threading.Thread.Sleep(1000);
      }
      while (rdr2.Read())
        Console.WriteLine("read one");
      cmd1.EndExecuteNonQuery(r);

      //rdr1.Close();
      rdr2.Read();
      rdr2.Close();
      //tx.Commit();
    }
  }
}
